/*
 * Copyright 2008 The Yacafe Project. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE YACAFE PROJECT ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE YACAFE PROJECT OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation are
 * those of the authors and should not be interpreted as representing official
 * policies, either expressed or implied, of the Yacafe Project.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "array.h"
#include "debug.h"

#define ARRAY_ALLOC_SIZE 128

struct array
{
	void **elem;
	int    len;
	int    size;
};

array_t *
array_create(void)
{
array_t *obj;

	obj = (array_t *)malloc(sizeof(array_t));

	if(obj)
	{
		obj->elem = NULL;
		obj->len  = 0;
		obj->size = 0;
	}

	return obj;
}

void
array_destroy(array_t *obj, void (*callback)(void *))
{
int i;

	if(!obj || !callback)
	{
		return;
	}

	for(i = 0; i < obj->len; i++)
	{
		callback(obj->elem[i]);
	}

	free(obj->elem);

	free(obj);
}

void *
array_add(array_t *obj, const void *data, void *(*callback)(const void *))
{
void *dst;
int   i;

	if(!obj)
	{
		goto ERROR;
	}

	if(obj->len >= obj->size)
	{
		obj->size += ARRAY_ALLOC_SIZE;

		obj->elem  = (void **)realloc(obj->elem, obj->size * sizeof(void *));

		if(!obj->elem)
		{
			goto ERROR;
		}

		for(i = (obj->size - ARRAY_ALLOC_SIZE); i < obj->size; i++)
		{
			obj->elem[i] = NULL;
		}
	}

	dst = (callback ? callback(data) : (void *)data);

	obj->elem[obj->len] = dst;

	obj->len += 1;

	return dst;

ERROR:
	return NULL;
}

void *
array_get(const array_t *obj, int offset)
{
	if(!obj)
	{
		goto ERROR;
	}

	if(offset < 0 || offset >= obj->len)
	{
		goto ERROR;
	}

	return obj->elem[offset];

ERROR:
	return NULL;
}

void *
array_set(array_t *obj, const void *data, int offset, void *(*set)(const void *), void (*destroy)(void *))
{
void *dst;

	if(!obj || !set || !destroy)
	{
		goto ERROR;
	}

	if(offset < 0 || offset >= obj->len)
	{
		goto ERROR;
	}

	destroy(obj->elem[offset]);

	dst = (set ? set(data) : (void *)data);

	obj->elem[offset] = dst;

	return dst;

ERROR:
	return NULL;
}

int
array_len(const array_t *obj)
{
	return obj ? obj->len : -1;
}

int
array_size(const array_t *obj)
{
	return obj ? obj->size : -1;
}
