/*
 * Copyright 2008 The Yacafe Project. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE YACAFE PROJECT ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE YACAFE PROJECT OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation are
 * those of the authors and should not be interpreted as representing official
 * policies, either expressed or implied, of the Yacafe Project.
 */

#ifndef __CONF_H__
#define __CONF_H__

#include "list.h"

typedef struct conf conf_t;
typedef struct conf_section conf_section_t;
typedef struct conf_section_item conf_section_item_t;

extern conf_t *
conf_create(void);
extern void
conf_destroy(conf_t *conf);
extern int
conf_numberof_sections(const conf_t *conf);
extern int
conf_load(conf_t *conf, const char *file);
extern int
conf_save(const conf_t *conf, const char *file);
extern conf_section_t *
conf_append_section(conf_t *conf, const char *name);
extern void
conf_remove_section(conf_t *conf, int index);
extern void
conf_remove_section_by_name(conf_t *conf, const char *name);
extern void
conf_remove_sections_by_name(conf_t *conf, const char *name);
extern conf_section_t *
conf_get_section(const conf_t *conf, int index);
extern conf_section_t *
conf_get_section_by_name(const conf_t *conf, const char *name);
extern list_t *
conf_get_sections_by_name(const conf_t *conf, const char *name);
extern int
conf_section_numberof_items(const conf_section_t *section);
extern char *
conf_section_get_name(const conf_section_t *section);
extern char *
conf_section_set_name(conf_section_t *section, const char *name);
extern conf_section_item_t *
conf_section_append_item(conf_section_t *section, const char *name, const char *value);
extern void
conf_section_remove_item(conf_section_t *section, int index);
extern void
conf_section_remove_item_by_name(conf_section_t *section, const char *name);
extern void
conf_section_remove_items_by_name(conf_section_t *section, const char *name);
extern conf_section_item_t *
conf_section_get_item(const conf_section_t *section, int index);
extern conf_section_item_t *
conf_section_get_item_by_name(const conf_section_t *section, const char *name);
extern list_t *
conf_section_get_items_by_name(const conf_section_t *section, const char *name);
extern char *
conf_section_item_get_name(const conf_section_item_t *item);
extern char *
conf_section_item_get_value(const conf_section_item_t *item);
extern char *
conf_section_item_set_name(conf_section_item_t *item, const char *name);
extern char *
conf_section_item_set_value(conf_section_item_t *item, const char *value);

#endif
