/*
 * Copyright 2008 The Yacafe Project. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE YACAFE PROJECT ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE YACAFE PROJECT OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation are
 * those of the authors and should not be interpreted as representing official
 * policies, either expressed or implied, of the Yacafe Project.
 */

#ifndef __FTP_H__
#define __FTP_H__

#include "list.h"
#include "logger.h"

typedef struct ftp ftp_t;

extern ftp_t *
ftp_create(void);
extern void
ftp_destroy(ftp_t *ftp);
extern int
ftp_open(ftp_t *ftp);
extern int
ftp_close(ftp_t *ftp);
extern char *
ftp_setopt_addr(ftp_t *ftp, const char *addr);
extern char *
ftp_setopt_port(ftp_t *ftp, const char *port);
extern char *
ftp_setopt_user(ftp_t *ftp, const char *user);
extern char *
ftp_setopt_pass(ftp_t *ftp, const char *pass);
extern logger_t *
ftp_setopt_logger(ftp_t *ftp, logger_t *logger);
extern int
ftp_cd(ftp_t *ftp, const char *path);
extern int
ftp_mkdir(ftp_t *ftp, const char *path);
extern int
ftp_cd_before_mkdir(ftp_t *ftp, const char *path);
extern char *
ftp_pwd(ftp_t *ftp);
extern list_t *
ftp_nlist(ftp_t *ftp, const char *path);
extern int
ftp_get(ftp_t *ftp, const char *remote, const char *local);
extern int
ftp_put(ftp_t *ftp, const char *local, const char *remote);

#endif
