/*
 * Copyright 2008 The Yacafe Project. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE YACAFE PROJECT ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE YACAFE PROJECT OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation are
 * those of the authors and should not be interpreted as representing official
 * policies, either expressed or implied, of the Yacafe Project.
 */

#ifndef __LOGGER_H__
#define __LOGGER_H__

#include <stdarg.h>

#define logger_entry_function(logger) _logger_entry_function(logger, __func__)
#define logger_leave_function(logger) _logger_leave_function(logger, __func__)

#define LOGGER_ERROR 1
#define LOGGER_WARNING 2
#define LOGGER_INFO 3
#define LOGGER_DEBUG 4
#define LOGGER_TRACE 5

typedef struct logger logger_t;

extern logger_t *
logger_create(void);
extern void
logger_destroy(logger_t *logger);
extern int
logger_open(logger_t *logger);
extern void
logger_close(logger_t *logger);
extern char *
logger_setopt_name(logger_t *logger, const char *name);
extern char *
logger_setopt_file(logger_t *logger, const char *file);
extern void
_logger_entry_function(logger_t *logger, const char *function);
extern void
_logger_leave_function(logger_t *logger, const char *function);
extern int
logger_printf(logger_t *logger, int level, const char *format,...);
extern void
logger_hexdump(logger_t *logger, int level, const void *data, int len);

#endif
