/*
 * Copyright 2008 The Yacafe Project. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE YACAFE PROJECT ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE YACAFE PROJECT OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation are
 * those of the authors and should not be interpreted as representing official
 * policies, either expressed or implied, of the Yacafe Project.
 */

#ifndef __PROPERTY_H__
#define __PROPERTY_H__

#include "list.h"

typedef struct property property_t;
typedef struct property_item property_item_t;

extern property_t *
property_create(void);
extern void
property_destroy(property_t *property);
extern int
property_numberof_items(const property_t *property);
extern int
property_load(property_t *property, const char *file);
extern int
property_save(const property_t *property, const char *file);
extern int
property_numberof_items(const property_t *property);
extern property_item_t *
property_append_item(property_t *property, const char *name, const char *value);
extern void
property_remove_item(property_t *property, int index);
extern void
property_remove_item_by_name(property_t *property, const char *name);
extern void
property_remove_items_by_name(property_t *property, const char *name);
extern property_item_t *
property_get_item(const property_t *property, int index);
extern property_item_t *
property_get_item_by_name(const property_t *property, const char *name);
extern list_t *
property_get_items_by_name(const property_t *property, const char *name);
extern char *
property_item_get_name(const property_item_t *item);
extern char *
property_item_get_value(const property_item_t *item);
extern char *
property_item_set_name(property_item_t *item, const char *name);
extern char *
property_item_set_value(property_item_t *item, const char *value);

#endif
