/*
 * Copyright 2008 The Yacafe Project. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE YACAFE PROJECT ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE YACAFE PROJECT OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation are
 * those of the authors and should not be interpreted as representing official
 * policies, either expressed or implied, of the Yacafe Project.
 */

#include <stdlib.h>
#include "strarray.h"
#include "strutil.h"
#include "array.h"
#include "debug.h"

struct strarray
{
	array_t *array;
};

static void
strarray_destroy_callback(void *data);
static void *
strarray_add_callback(const void *data);

strarray_t *
strarray_create(void)
{
	return (strarray_t *)array_create();
}

void
strarray_destroy(strarray_t *obj)
{
	array_destroy((array_t *)obj, strarray_destroy_callback);
}

static void
strarray_destroy_callback(void *data)
{
	free(data);
}

char *
strarray_add(strarray_t *obj, const char *data)
{
	return (char *)array_add((array_t *)obj, (const void *)data, strarray_add_callback);
}

static void *
strarray_add_callback(const void *data)
{
	return data ? strutil_dup(data) : NULL;
}

char *
strarray_get(const strarray_t *obj, int offset)
{
	return (char *)array_get((const array_t *)obj, offset);
}

char *
strarray_set(strarray_t *obj, const char *data, int offset)
{
	return (char *)array_set((array_t *)obj, (const void *)data, offset, strarray_add_callback, strarray_destroy_callback);
}

int
strarray_len(const strarray_t *obj)
{
	return array_len((const array_t *)obj);
}

int
strarray_size(const strarray_t *obj)
{
	return array_size((const array_t *)obj);
}
