/*
 * Copyright 2008 The Yacafe Project. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE YACAFE PROJECT ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE YACAFE PROJECT OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation are
 * those of the authors and should not be interpreted as representing official
 * policies, either expressed or implied, of the Yacafe Project.
 */

#include <stdlib.h>
#include <string.h>
#include "xbinary.h"
#include "debug.h"

#define XBINARY_ALLOC 128

struct xbinary
{
	char *data;
	int allocate;
	int size;
};

xbinary_t *
xbinary_create(void)
{
	xbinary_t *xbinary;

	xbinary = (xbinary_t *)malloc(sizeof(xbinary_t));

	if(xbinary)
	{
		xbinary->data = NULL;
		xbinary->allocate = 0;
		xbinary->size = 0;
	}

	return xbinary;
}

void
xbinary_destroy(xbinary_t *xbinary)
{
	if(xbinary)
	{
		free(xbinary->data);
	}

	free(xbinary);
}

char *
xbinary_concat(xbinary_t *xbinary, const char *data, int size)
{
	int total;

	if(!xbinary || size < 0)
	{
		goto ERROR;
	}

	total = xbinary->size + size;

	if(total >= xbinary->allocate)
	{
		xbinary->allocate += XBINARY_ALLOC * ((total / XBINARY_ALLOC) + 1);

		xbinary->data = (char *)realloc(xbinary->data, xbinary->allocate);

		if(!xbinary->data)
		{
			goto ERROR;
		}

		memset(xbinary->data + xbinary->size, 0x00, xbinary->allocate - xbinary->size);
	}

	memcpy(xbinary->data + xbinary->size, data, size);

	xbinary->size += size;

	return xbinary->data;

ERROR:
	return NULL;
}

char *
xbinary_raw(xbinary_t *xbinary)
{
	return xbinary ? xbinary->data : NULL;
}

int
xbinary_size(xbinary_t *xbinary)
{
	return xbinary ? xbinary->size : -1;
}
