/*
 * Copyright 2008 The Yacafe Project. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE YACAFE PROJECT ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE YACAFE PROJECT OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation are
 * those of the authors and should not be interpreted as representing official
 * policies, either expressed or implied, of the Yacafe Project.
 */

#include <stdlib.h>
#include <string.h>
#include "xstring.h"
#include "debug.h"

#define XSTRING_ALLOC 128

struct xstring
{
	char *str;
	int size;
	int len;
};

xstring_t *
xstring_create(const char *str)
{
	xstring_t *xstring;

	xstring = (xstring_t *)malloc(sizeof(xstring_t));

	if(xstring)
	{
		xstring->str = NULL;
		xstring->size = 0;
		xstring->len = 0;

		xstring_concat(xstring, str, NULL);
	}

	return xstring;
}

void
xstring_destroy(xstring_t *xstring)
{
	if(xstring)
	{
		free(xstring->str);
	}

	free(xstring);
}

char *
xstring_concat(xstring_t *xstring, const char *str, const char *delm)
{
	int dlen, slen, len;

	if(!xstring)
	{
		goto ERROR;
	}

	dlen = delm ? strlen(delm) : 0;
	slen = str ? strlen(str) : 0;

	len = xstring->len + dlen + slen;

	if(len >= xstring->size)
	{
		xstring->size += XSTRING_ALLOC * ((len / XSTRING_ALLOC) + 1);

		xstring->str = (char *)realloc(xstring->str, xstring->size);

		if(!xstring->str)
		{
			goto ERROR;
		}

		memset(xstring->str + xstring->len, 0x00, xstring->size - xstring->len);
	}

	if(delm)
	{
		memcpy(xstring->str + xstring->len, delm, dlen);

		xstring->len += dlen;
	}

	if(str)
	{
		memcpy(xstring->str + xstring->len, str, slen);

		xstring->len += slen;
	}

	return xstring->str;

ERROR:
	return NULL;
}

char *
xstring_raw(xstring_t *xstring)
{
	return xstring ? xstring->str : NULL;
}
