/*
 * Copyright 2008 The Yacafe Project. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE YACAFE PROJECT ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE YACAFE PROJECT OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation are
 * those of the authors and should not be interpreted as representing official
 * policies, either expressed or implied, of the Yacafe Project.
 */

#ifndef __REQUEST_H__
#define __REQUEST_H__

#include "list.h"

typedef struct request request_t;
typedef struct request_item request_item_t;
typedef struct request_upload_item request_upload_item_t;

extern request_t *
request_create(void);
extern void
request_destroy(request_t *request);
extern int
request_numberof_items(const request_t *request);
extern request_item_t *
request_append_item(request_t *request, const char *name, const char *value);
extern void
request_remove_item(request_t *request, int index);
extern void
request_remove_item_by_name(request_t *request, const char *name);
extern void
request_remove_items_by_name(request_t *request, const char *name);
extern request_item_t *
request_get_item(const request_t *request, int index);
extern request_item_t *
request_get_item_by_name(const request_t *request, const char *name);
extern list_t *
request_get_items_by_name(const request_t *request, const char *name);
extern int
request_numberof_upload_items(const request_t *request);
extern request_upload_item_t *
request_append_upload_item(request_t *request, const char *name);
extern void
request_remove_upload_item(request_t *request, int index);
extern void
request_remove_upload_item_by_name(request_t *request, const char *name);
extern void
request_remove_upload_items_by_name(request_t *request, const char *name);
extern request_upload_item_t *
request_get_upload_item(const request_t *request, int index);
extern request_upload_item_t *
request_get_upload_item_by_name(const request_t *request, const char *name);
extern list_t *
request_get_upload_items_by_name(const request_t *request, const char *name);
extern char *
request_item_get_name(const request_item_t *item);
extern char *
request_item_get_value(const request_item_t *item);
extern char *
request_item_set_value(request_item_t *item, const char *value);
extern char *
request_upload_item_get_name(const request_upload_item_t *item);
extern char *
request_upload_item_get_path(const request_upload_item_t *item);
extern char *
request_upload_item_set_path(request_upload_item_t *item, const char *path);

#endif
