/*
 * Copyright 2008 The Yacafe Project. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE YACAFE PROJECT ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE YACAFE PROJECT OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * The views and conclusions contained in the software and documentation are
 * those of the authors and should not be interpreted as representing official
 * policies, either expressed or implied, of the Yacafe Project.
 */

#ifndef __RESPONSE_H__
#define __RESPONSE_H__

#include "list.h"

typedef struct response response_t;
typedef struct response_node response_node_t;
typedef struct response_node_property response_node_property_t;

extern response_t *
response_create(void);
extern void
response_destroy(response_t *response);
extern int
response_numberof_nodes(const response_t *response);
extern response_node_t *
response_append_node(response_t *response, const char *name);
extern void
response_remove_node(response_t *response, int index);
extern void
response_remove_node_by_name(response_t *response, const char *name);
extern void
response_remove_nodes_by_name(response_t *response, const char *name);
extern response_node_t *
response_get_node(const response_t *response, int index);
extern response_node_t *
response_get_node_by_name(const response_t *response, const char *name);
extern list_t *
response_get_nodes_by_name(const response_t *response, const char *name);
extern int
response_node_numberof_nodes(const response_node_t *node);
extern int
response_node_numberof_properties(const response_node_t *node);
extern char *
response_node_get_name(const response_node_t *node);
extern char *
response_node_set_name(response_node_t *node, const char *name);
extern response_node_t *
response_node_append_node(response_node_t *node, const char *name);
extern response_node_property_t *
response_node_append_property(response_node_t *node, const char *name, const char *value);
extern void
response_node_remove_node(response_node_t *node, int index);
extern void
response_node_remove_property(response_node_t *node, int index);
extern void
response_node_remove_node_by_name(response_node_t *node, const char *name);
extern void
response_node_remove_property_by_name(response_node_t *node, const char *name);
extern void
response_node_remove_nodes_by_name(response_node_t *node, const char *name);
extern void
response_node_remove_properties_by_name(response_node_t *node, const char *name);
extern response_node_t *
response_node_get_node(response_node_t *node, int index);
extern response_node_property_t *
response_node_get_property(response_node_t *node, int index);
extern response_node_t *
response_node_get_node_by_name(response_node_t *node, const char *name);
extern response_node_property_t *
response_node_get_property_by_name(response_node_t *node, const char *name);
extern list_t *
response_node_get_nodes_by_name(response_node_t *node, const char *name);
extern list_t *
response_node_get_properties_by_name(response_node_t *node, const char *name);
extern char *
response_node_property_get_name(const response_node_property_t *property);
extern char *
response_node_property_get_value(const response_node_property_t *property);
extern char *
response_node_property_set_name(response_node_property_t *property, const char *name);
extern char *
response_node_property_set_value(response_node_property_t *property, const char *value);

#endif
